
library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.NUMERIC_STD.all;


entity sobelProc is
    port (I_clk          : in  std_logic;
          I_rst          : in  std_logic;
          I_go           : in  std_logic;
          -- interface avec la mémoire IN (lecture)
          O_enM_R        : out std_logic;
          O_ADR_R        : out std_logic_vector (13 downto 0);
          I_pixel        : in  std_logic_vector (7 downto 0);  -- Pixel from memory IN
          -- interface avec la mémoire OUT (écriture)
          O_enM_W        : out std_logic;
          O_ADR_W        : out std_logic_vector (13 downto 0);
          O_pixEdge      : out std_logic;                      -- Edge to memory OUT
          -- signal de commande vers le contrôleur VGA
          O_StartDisplay : out std_logic
          );
end sobelProc;


architecture Behavioral of sobelProc is

-- déclaration des sous-composants

    -- unité opérative
    component operativeUnit is
        port (I_clk       : in  std_logic;
              I_pixel     : in  std_logic_vector (7 downto 0);  -- Pixel from the memory
              I_ldPix11   : in  std_logic;
              I_ldPix21   : in  std_logic;
              I_ldPix31   : in  std_logic;
              I_shReg     : in  std_logic;
              I_ldPixEdge : in  std_logic;
              O_pixEdge   : out std_logic
              );
    end component operativeUnit;

    -- générateur d'adresses
    component adrgenUnit is
        port (I_clk         : in  std_logic;
              I_clr_PtrLine : in  std_logic;
              I_inc_PtrLine : in  std_logic;
              I_clr_PtrCol  : in  std_logic;
              I_inc_PtrCol  : in  std_logic;
              I_selPix      : in  std_logic_vector (1 downto 0);
              O_EndImage    : out std_logic;
              O_NewLine     : out std_logic;
              O_ADR_R       : out std_logic_vector (13 downto 0);
              O_ADR_W       : out std_logic_vector (13 downto 0)
              );
    end component adrgenUnit;

    -- automate
    component automate is
        port (I_clk          : in  std_logic;
              I_rst          : in  std_logic;
              I_go           : in  std_logic;
              I_EndImage     : in  std_logic;
              I_NewLine      : in  std_logic;
              -- signaux de commandes vers l'unité opérative
              O_ldPix11      : out std_logic;
              O_ldPix21      : out std_logic;
              O_ldPix31      : out std_logic;
              O_shReg        : out std_logic;
              O_ldPixEdge    : out std_logic;
              -- signaux de commandes vers le générateur d'adresses
              O_clr_PtrLine  : out std_logic;
              O_inc_PtrLine  : out std_logic;
              O_clr_PtrCol   : out std_logic;
              O_inc_PtrCol   : out std_logic;
              O_selPix       : out std_logic_vector (1 downto 0);
              -- signaux de commandes vers les mémoires
              O_enM_R        : out std_logic;
              O_enM_W        : out std_logic;
              -- signal de commande vers le contrôleur VGA
              O_StartDisplay : out std_logic
              );
    end component automate;

-- déclaration des signaux internes
    signal S_EndImage    : std_logic;
    signal S_NewLine     : std_logic;
    signal S_ldPix11     : std_logic;
    signal S_ldPix21     : std_logic;
    signal S_ldPix31     : std_logic;
    signal S_shReg       : std_logic;
    signal S_ldPixEdge   : std_logic;
    signal S_clr_PtrLine : std_logic;
    signal S_inc_PtrLine : std_logic;
    signal S_clr_PtrCol  : std_logic;
    signal S_inc_PtrCol  : std_logic;
    signal S_selPix      : std_logic_vector (1 downto 0);


begin

-- instanciation des sous-composants et établissement des interconnexions

    -- instanciation de l'unité opérative
    operativeUnit_1 : entity work.operativeUnit
        port map (
            I_clk       => I_clk,
            I_pixel     => I_pixel,
            I_ldPix11   => S_ldPix11,
            I_ldPix21   => S_ldPix21,
            I_ldPix31   => S_ldPix31,
            I_shReg     => S_shReg,
            I_ldPixEdge => S_ldPixEdge,
            O_pixEdge   => O_pixEdge
            );

    -- instanciation du générateur d'adresses
    adrgenUnit_1 : entity work.adrgenUnit
        port map (
            I_clk         => I_clk,
            I_clr_PtrLine => S_clr_PtrLine,
            I_inc_PtrLine => S_inc_PtrLine,
            I_clr_PtrCol  => S_clr_PtrCol,
            I_inc_PtrCol  => S_inc_PtrCol,
            I_selPix      => S_selPix,
            O_EndImage    => S_EndImage,
            O_NewLine     => S_NewLine,
            O_ADR_R       => O_ADR_R,
            O_ADR_W       => O_ADR_W
            );

    -- instanciation de l'automate
    automate_1 : entity work.automate
        port map (
            I_clk          => I_clk,
            I_rst          => I_rst,
            I_go           => I_go,
            I_EndImage     => S_EndImage,
            I_NewLine      => S_NewLine,
            O_ldPix11      => S_ldPix11,
            O_ldPix21      => S_ldPix21,
            O_ldPix31      => S_ldPix31,
            O_shReg        => S_shReg,
            O_ldPixEdge    => S_ldPixEdge,
            O_clr_PtrLine  => S_clr_PtrLine,
            O_inc_PtrLine  => S_inc_PtrLine,
            O_clr_PtrCol   => S_clr_PtrCol,
            O_inc_PtrCol   => S_inc_PtrCol,
            O_selPix       => S_selPix,
            O_enM_R        => O_enM_R,
            O_enM_W        => O_enM_W,
            O_StartDisplay => O_StartDisplay
            );

end Behavioral;
